//------------------------------------------------------------------------------
// Copyright (c) 2008 www.dnaide.com
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;

using DNAide.Core;

namespace DNAide.Collections
{
	/// <summary>
	/// Queue helper methods
	/// </summary>
	public static partial class QueueAide
	{
		/// <summary>
		/// Returns a Queue&lt;T&gt; from a comma separated string.
		/// Trims spaces, does not allow duplicates and ignores empty entities
		/// </summary>
		/// <param name="value">value</param>
		/// <returns>Queue&lt;T&gt;</returns>
		public static Queue<T> GetQueueAs<T>(String value)
		{
			return GetQueueAs<T>(value, new char[] { ',' }, true, false, StringSplitOptions.RemoveEmptyEntries);
		}


		/// <summary>
		/// Returns a Queue&lt;T&gt; from a comma separated string.
		/// Trims spaces, does not allow duplicates and ignores empty entities
		/// </summary>
		/// <param name="value">value</param>
		/// <param name="delimeter">delimeter</param>
		/// <returns>Queue&lt;T&gt;</returns>
		public static Queue<T> GetQueueAs<T>(String value, Char[] delimeter)
		{
			return GetQueueAs<T>(value, delimeter, true, false, StringSplitOptions.RemoveEmptyEntries);
		}


		/// <summary>
		/// Returns a Queue&lt;T&gt; from a delimetered string.
		/// Does not allow duplicates and ignores empty entities
		/// </summary>
		/// <param name="value">value</param>
		/// <param name="delimeter">delimeter</param>
		/// <param name="trimSpaces">trimSpaces</param>
		/// <returns>Queue&lt;T&gt;</returns>
		public static Queue<T> GetQueueAs<T>(String value, Char[] delimeter, Boolean trimSpaces)
		{
			return GetQueueAs<T>(value, delimeter, trimSpaces, false, StringSplitOptions.RemoveEmptyEntries);
		}


		/// <summary>
		/// Returns a Queue&lt;T&gt; from a delimetered string.
		/// Ignores empty entities
		/// </summary>
		/// <param name="value">value</param>
		/// <param name="delimeter">delimeter</param>
		/// <param name="trimSpaces">trimSpaces</param>
		/// <param name="allowDuplicates">allowDuplicates</param>
		/// <returns>Queue&lt;T&gt;</returns>
		public static Queue<T> GetQueueAs<T>(String value, Char[] delimeter, Boolean trimSpaces, Boolean allowDuplicates)
		{
			return GetQueueAs<T>(value, delimeter, trimSpaces, allowDuplicates, StringSplitOptions.RemoveEmptyEntries);
		}


		/// <summary>
		/// Returns a Queue&lt;T&gt; from a delimetered string.
		/// </summary>
		/// <param name="value">value</param>
		/// <param name="delimeter">delimeter</param>
		/// <param name="trimSpaces">trimSpaces</param>
		/// <param name="allowDuplicates">allowDuplicates</param>
		/// <param name="splitOptions">splitOptions</param>
		/// <returns>Queue&lt;T&gt;</returns>
		public static Queue<T> GetQueueAs<T>(String value, Char[] delimeter, Boolean trimSpaces, Boolean allowDuplicates, StringSplitOptions splitOptions)
		{
			Queue<T> retval = new Queue<T>();

			if (string.IsNullOrEmpty(value))
				return retval;

			if (delimeter == null)
				throw new ArgumentException("delimeter");

			string[] temp = value.Split(delimeter, splitOptions);
			for (int i = 0; i < temp.Length; i++)
			{
				T valueToAdd = CastAide.As<T>(temp[i]);

				if (trimSpaces)
					valueToAdd = CastAide.As<T>(temp[i].Trim());

				if (!allowDuplicates)
				{
					if (retval.Contains(valueToAdd))
						continue;
				}

				retval.Enqueue(valueToAdd);
			}

			return retval;
		}
	}
}
