//------------------------------------------------------------------------------
// Copyright (c) 2008 www.dnaide.com
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;
using System.Web;

using DNAide.Core;

namespace DNAide.Configuration
{
	/// <summary>
	/// Property value that will inspect x.config for stored value
	/// and return if found, otherwise a default instance of T 
	/// or an instance of T created via a delegate.
	/// </summary>
	/// <typeparam name="T"></typeparam>
	public partial class ConfigProperty<T>
	{
		private readonly string _key;
		private T _default;
		private readonly Creator<T> _creator;
		private readonly object _lock = new object();
		private bool _created;

		/// <summary>
		/// Key to reference value in config.
		/// </summary>
		public string Key
		{
			get { return _key; }
		}

		/// <summary>
		/// Config value if found, otherwise instance from creator
		/// if available or default value.
		/// </summary>
		public T Value
		{
			get
			{
				if (ConfigurationManager.AppSettings[_key] == null)
				{
					if (_creator != null && !_created)
					{
						lock (_lock)
						{
							_default = _creator();
							_created = true;
						}
					}

					return _default;
				}
				else
					return CastAide.As(ConfigurationManager.AppSettings[_key], _default);
			}
		}

		/// <summary>
		/// Will use default value of type.
		/// </summary>
		/// <param name="key">Key to reference in config</param>
		public ConfigProperty(string key)
		{
			_key = key;
			_default = default(T);
		}

		/// <summary>
		/// Will use specified default value.
		/// </summary>
		/// <param name="key">Key to reference in config</param>
		/// <param name="defaultValue">Default value to use if config empty</param>
		public ConfigProperty(string key, T defaultValue)
		{
			_key = key;
			_default = defaultValue;
		}

		/// <summary>
		/// Will use specified creator delegate
		/// </summary>
		/// <param name="key">Key to reference in config</param>
		/// <param name="creator">Default instance as supplied by creator delegate</param>
		public ConfigProperty(string key, Creator<T> creator)
		{
			_key = key;
			_creator = creator;
		}
	}
}
