//------------------------------------------------------------------------------
// Copyright (c) 2008 www.dnaide.com
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Globalization;

namespace DNAide.Core
{
	public static partial class CastAide
	{

		/// <summary>
		/// Returns an object as a Boolean.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <returns>Boolean</returns>
		/// <remarks>
		/// <para>
		/// Will return false if the cast fails.
		/// </para>
		/// <para>
		/// Uses CultureInfo.CurrentCulture.
		/// </para>
		/// </remarks>
		public static Boolean AsBoolean(object value)
		{
			return AsBoolean(value, false, CultureInfo.CurrentCulture);
		}


		/// <summary>
		/// Returns an object as a Boolean.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <returns>Boolean</returns>
		/// <remarks>
		/// <para>
		/// Uses CultureInfo.CurrentCulture.
		/// </para>
		/// </remarks>
		public static Boolean AsBoolean(object value, Boolean defaultValue)
		{
			return AsBoolean(value, defaultValue, CultureInfo.CurrentCulture);
		}


		/// <summary>
		/// Returns an object as a Boolean.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="provider">Instance of IFormatProvider</param>
		/// <returns>Boolean</returns>
		/// <remarks>
		/// <para>
		/// Will return false if the cast fails.
		/// </para>
		/// </remarks>
		public static Boolean AsBoolean(object value, IFormatProvider provider)
		{
			return AsBoolean(value, false, provider);
		}


		/// <summary>
		/// Returns an object as a Boolean.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <param name="provider">Instance of IFormatProvider</param>
		/// <returns>Boolean</returns>
		public static Boolean AsBoolean(object value, Boolean defaultValue, IFormatProvider provider)
		{
			if (value == null || value == System.DBNull.Value)
				return defaultValue;

			if (provider == null)
				provider = CultureInfo.CurrentCulture;

			Boolean retval;
			try
			{
				retval = Convert.ToBoolean(value, provider);
			}
			catch (InvalidCastException)
			{
				retval = defaultValue;
			}
			catch (FormatException)
			{
				retval = defaultValue;
			}
			return retval;
		}

		
		/// <summary>
		/// Returns an object as a Boolean using the invariant culture.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <returns>Boolean</returns>
		/// <remarks>
		/// <para>
		/// Will return false if the cast fails.
		/// </para>
		/// </remarks>
		public static Boolean AsBooleanInvariant(object value)
		{
			return AsBoolean(value, false, CultureInfo.InvariantCulture);
		}

		
		/// <summary>
		/// Returns an object as a Boolean using the invariant culture.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <returns>Boolean</returns>
		public static Boolean AsBooleanInvariant(object value, Boolean defaultValue)
		{
			return AsBoolean(value, defaultValue, CultureInfo.InvariantCulture);
		}

	}
}
