//------------------------------------------------------------------------------
// Copyright (c) 2008 www.dnaide.com
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Globalization;

namespace DNAide.Core
{
	public static partial class CastAide
	{

		/// <summary>
		/// Returns an object as a Char.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <returns>Char</returns>
		/// <remarks>
		/// <para>
		/// Will return ' ' if the cast fails.
		/// </para>
		/// <para>
		/// Uses CultureInfo.CurrentCulture.
		/// </para>
		/// </remarks>
		public static Char AsChar(object value)
		{
			return AsChar(value, ' ', CultureInfo.CurrentCulture);
		}


		/// <summary>
		/// Returns an object as a Char.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <returns>Char</returns>
		/// <remarks>
		/// <para>
		/// Uses CultureInfo.CurrentCulture.
		/// </para>
		/// </remarks>
		public static Char AsChar(object value, Char defaultValue)
		{
			return AsChar(value, defaultValue, CultureInfo.CurrentCulture);
		}


		/// <summary>
		/// Returns an object as a Char.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="provider">Instance of IFormatProvider</param>
		/// <returns>Char</returns>
		/// <remarks>
		/// <para>
		/// Will return ' ' if the cast fails.
		/// </para>
		/// </remarks>
		public static Char AsChar(object value, IFormatProvider provider)
		{
			return AsChar(value, ' ', provider);
		}


		/// <summary>
		/// Returns an object as a Char.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <param name="provider">Instance of IFormatProvider</param>
		/// <returns>Char</returns>
		public static Char AsChar(object value, Char defaultValue, IFormatProvider provider)
		{
			if (value == null || value == System.DBNull.Value)
				return defaultValue;

			if (provider == null)
				provider = CultureInfo.CurrentCulture;

			Char retval;

			try
			{
				retval = Convert.ToChar(value, provider);
			}
			catch (InvalidCastException)
			{
				retval = defaultValue;
			}
			catch (FormatException)
			{
				retval = defaultValue;
			}
			return retval;
		}


		/// <summary>
		/// Returns an object as a Char using the invariant culture.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <returns>Char</returns>
		/// <remarks>
		/// <para>
		/// Will return ' ' if the cast fails.
		/// </para>
		/// </remarks>
		public static Char AsCharInvariant(object value)
		{
			return AsChar(value, ' ', CultureInfo.InvariantCulture);
		}


		/// <summary>
		/// Returns an object as a Char using the invariant culture.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <returns>Char</returns>
		public static Char AsCharInvariant(object value, Char defaultValue)
		{
			return AsChar(value, defaultValue, CultureInfo.InvariantCulture);
		}

	}
}
