//------------------------------------------------------------------------------
// Copyright (c) 2008 www.dnaide.com
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Globalization;

namespace DNAide.Core
{
	public static partial class CastAide
	{

		/// <summary>
		/// Returns an object as a Double.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <returns>Double</returns>
		/// <remarks>
		/// <para>
		/// Will return 0 if the cast fails.
		/// </para>
		/// <para>
		/// Uses CultureInfo.CurrentCulture.
		/// </para>
		/// </remarks>
		public static Double AsDouble(object value)
		{
			return AsDouble(value, 0, CultureInfo.CurrentCulture);
		}


		/// <summary>
		/// Returns an object as a Double.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <returns>Double</returns>
		/// <remarks>
		/// <para>
		/// Uses CultureInfo.CurrentCulture.
		/// </para>
		/// </remarks>
		public static Double AsDouble(object value, Double defaultValue)
		{
			return AsDouble(value, defaultValue, CultureInfo.CurrentCulture);
		}


		/// <summary>
		/// Returns an object as a Double.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="provider">Instance of IFormatProvider</param>
		/// <returns>Double</returns>
		/// <remarks>
		/// <para>
		/// Will return 0 if the cast fails.
		/// </para>
		/// </remarks>
		public static Double AsDouble(object value, IFormatProvider provider)
		{
			return AsDouble(value, 0, provider);
		}


		/// <summary>
		/// Returns an object as a Double.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <param name="provider">Instance of IFormatProvider</param>
		/// <returns>Double</returns>
		public static Double AsDouble(object value, Double defaultValue, IFormatProvider provider)
		{
			if (value == null || value == System.DBNull.Value)
				return defaultValue;

			if (provider == null)
				provider = CultureInfo.CurrentCulture;

			Double retval;

			try
			{
				retval = Convert.ToDouble(value, provider);
			}
			catch (InvalidCastException)
			{
				retval = defaultValue;
			}
			catch (FormatException)
			{
				retval = defaultValue;
			}
			return retval;
		}


		/// <summary>
		/// Returns an object as a Double using the invariant culture.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <returns>Double</returns>
		/// <remarks>
		/// <para>
		/// Will return 0 if the cast fails.
		/// </para>
		/// </remarks>
		public static Double AsDoubleInvariant(object value)
		{
			return AsDouble(value, 0, CultureInfo.InvariantCulture);
		}


		/// <summary>
		/// Returns an object as a Double using the invariant culture.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <returns>Double</returns>
		public static Double AsDoubleInvariant(object value, Double defaultValue)
		{
			return AsDouble(value, defaultValue, CultureInfo.InvariantCulture);
		}

	}
}
