//------------------------------------------------------------------------------
// Copyright (c) 2007 Ste Brennan (dnaide.com)
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;
using System.Web;

using DNAide.Core;

namespace DNAide.Web
{
	/// <summary>
	/// Property value that will inspect the request cookies for value
	/// and return if found, otherwise a default instance of T 
	/// or an instance of T created via a delegate.
	/// </summary>
	/// <typeparam name="T"></typeparam>
	public partial class CookieProperty<T>
	{
		private readonly string _key;
		private readonly T _default;
		private readonly Creator<T> _creator;

		/// <summary>
		/// Key to reference cookie.
		/// </summary>
		public string Key
		{
			get { return _key; }
		}

		/// <summary>
		/// Cookie value if found, otherwise instance from creator
		/// if available or default value.
		/// </summary>
		public T Value
		{
			get
			{
				if (HttpContext.Current.Request.Cookies[_key] == null)
				{
					if (_creator != null)
					{
						T result = _creator();
						Value = result;
						return result;
					}
					else
						return _default;
				}
				else
					return CastAide.As(HttpContext.Current.Request.Cookies[_key].Value, _default);
			}
			set
			{
				HttpCookie cookie = new HttpCookie(_key);

				// No support for date expiry of other cookie settings at present
				cookie.Value = CastAide.AsString(value);

				HttpContext.Current.Request.Cookies.Add(cookie);
			}
		}

		/// <summary>
		/// Will use default value of type.
		/// </summary>
		/// <param name="key">Key to reference in cookie collection</param>
		public CookieProperty(string key)
		{
			_key = key;
			_default = default(T);
		}

		/// <summary>
		/// Will use specified default value.
		/// </summary>
		/// <param name="key">Key to reference in cookie collection</param>
		/// <param name="defaultValue">Default value to use if cookie value empty or cannot be found</param>
		public CookieProperty(string key, T defaultValue)
		{
			_key = key;
			_default = defaultValue;
		}

		/// <summary>
		/// Will use specified creator delegate
		/// </summary>
		/// <param name="key">Key to reference in cookie collection</param>
		/// <param name="creator">Default instance as supplied by creator delegate</param>
		public CookieProperty(string key, Creator<T> creator)
		{
			_key = key;
			_creator = creator;
		}
	}
}
