//------------------------------------------------------------------------------
// Copyright (c) 2007 Ste Brennan (dnaide.com)
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.Web;

using DNAide.Core;

namespace DNAide.Web
{
	/// <summary>
	/// Property value that will inspect session for stored value
	/// and return if found, otherwise a default instance of T 
	/// or an instance of T created via a delegate.
	/// </summary>
	/// <typeparam name="T">class</typeparam>
	public partial class SessionProperty<T>
	{
		private readonly string _key;
		private readonly T _default;
		private readonly Creator<T> _creator;

		/// <summary>
		/// Key to reference value in session.
		/// </summary>
		public string Key
		{
			get { return _key; }
		}

		/// <summary>
		/// Stored value if found, otherwise instance from creator
		/// if available or default value.
		/// </summary>
		public T Value
		{
			get
			{
				if (HttpContext.Current.Session[_key] == null)
				{
					if (_creator != null)
					{
						T result = _creator();
						Value = result;
						return result;
					}
					else
						return _default;
				}
				else
					return (T)HttpContext.Current.Session[_key];
			}
			set
			{
				HttpContext.Current.Session[_key] = value;
			}
		}

		/// <summary>
		/// Will use default value of type.
		/// </summary>
		/// <param name="key">Key to reference in session</param>
		public SessionProperty(string key)
		{
			_key = key;
			_default = default(T);
		}

		/// <summary>
		/// Will use specified default value.
		/// </summary>
		/// <param name="key">Key to reference in session</param>
		/// <param name="defaultValue">Default value to use if session empty</param>
		public SessionProperty(string key, T defaultValue)
		{
			_key = key;
			_default = defaultValue;
		}

		/// <summary>
		/// Will use specified creator delegate
		/// </summary>
		/// <param name="key">Key to reference in session</param>
		/// <param name="creator">Default instance as supplied by creator delegate</param>
		public SessionProperty(string key, Creator<T> creator)
		{
			_key = key;
			_creator = creator;
		}
	}
}
